@@ -386,6 +386,66 @@ const Pulsar = require('../index.js');
       await client.close();
     });
 
+    test('Message Listener error handling', async () => {
+      const client = new Pulsar.Client({
+        serviceUrl: 'pulsar://localhost:6650',
+      });
+      let syncFinsh;
+      const syncPromise = new Promise((resolve) => {
+        syncFinsh = resolve;
+      });
+      let asyncFinsh;
+      const asyncPromise = new Promise((resolve) => {
+        asyncFinsh = resolve;
+      });
+      Pulsar.Client.setLogHandler((level, file, line, message) => {
+        if (level === 3) { // should be error level
+          if (message.includes('consumer1 callback expected error')) {
+            syncFinsh();
+          }
+          if (message.includes('consumer2 callback expected error')) {
+            asyncFinsh();
+          }
+        }
+      });
+
+      const topic = 'test-error-listener';
+      const producer = await client.createProducer({
+        topic,
+        batchingEnabled: false,
+      });
+
+      await producer.send('test-message');
+
+      const consumer1 = await client.subscribe({
+        topic,
+        subscription: 'sync',
+        subscriptionType: 'Shared',
+        subscriptionInitialPosition: 'Earliest',
+        listener: (message, messageConsumer) => {
+          throw new Error('consumer1 callback expected error');
+        },
+      });
+
+      const consumer2 = await client.subscribe({
+        topic,
+        subscription: 'async',
+        subscriptionType: 'Shared',
+        subscriptionInitialPosition: 'Earliest',
+        listener: async (message, messageConsumer) => {
+          throw new Error('consumer2 callback expected error');
+        },
+      });
+
+      await syncPromise;
+      await asyncPromise;
+
+      await consumer1.close();
+      await consumer2.close();
+      await producer.close();
+      await client.close();
+    });
+
     test('acknowledgeCumulative', async () => {
       const client = new Pulsar.Client({
         serviceUrl: 'pulsar://localhost:6650',
@@ -929,6 +989,47 @@ const Pulsar = require('../index.js');
       await consumer.close();
       await client.close();
     });
+    test('Basic produce and read encryption', async () => {
+      const client = new Pulsar.Client({
+        serviceUrl: 'pulsar://localhost:6650',
+        operationTimeoutSeconds: 30,
+      });
+
+      const topic = 'persistent://public/default/encryption-produce-read';
+      const producer = await client.createProducer({
+        topic,
+        sendTimeoutMs: 30000,
+        batchingEnabled: true,
+        publicKeyPath: `${__dirname}/certificate/public-key.client-rsa.pem`,
+        encryptionKey: 'encryption-key',
+      });
+
+      const reader = await client.createReader({
+        topic,
+        startMessageId: Pulsar.MessageId.earliest(),
+        privateKeyPath: `${__dirname}/certificate/private-key.client-rsa.pem`,
+      });
+
+      const messages = [];
+      for (let i = 0; i < 10; i += 1) {
+        const msg = `my-message-${i}`;
+        producer.send({
+          data: Buffer.from(msg),
+        });
+        messages.push(msg);
+      }
+      await producer.flush();
+
+      const results = [];
+      for (let i = 0; i < 10; i += 1) {
+        const msg = await reader.readNext();
+        results.push(msg.getData().toString());
+      }
+      expect(lodash.difference(messages, results)).toEqual([]);
+      await producer.close();
+      await reader.close();
+      await client.close();
+    });
     test('Produce/Consume/Read/IsConnected', async () => {
       const client = new Pulsar.Client({
         serviceUrl: 'pulsar://localhost:6650',
